; Periodic Table v1.2 for Ash 3.0 (c) 1997-1998 Matt Maurano <mtm@maurano.com>

#include "ti82.h"

.org START_ADDR

.db	"Per. Table",0

CurStAddr	=	TEXT_MEM

	call SetGraph

	ld hl, Data
	ld (CurStAddr), hl

	call DispMenu

KeyLoop:
	ROM_CALL(KEY_HAND)	; Check for keys. Key_Hand allows the contrast to be changed while in the loop. With Ash's relocation, we will crash if 2nd+On is pressed

	cp $02			; Is it the key code for left?
	jr z, Prev			; If it is, print the previous entry
	cp $01			; Is it the key code for right?
	jr z, Next			; If it is, print the next entry
	cp $06			; Is it the key code for clear?
	jr nz, KeyLoop		; If it's not, loop until it is pressed. If it is, quit. (The ret at the end of setgraph will quit the program. It saves a couple bytes to use an existing ret than to make a new one)

SetGraph:				; Since this large amount of code is used twice, its smaller to do a function
	ROM_CALL(CLEARLCD)	; Clear the screen

	ld hl, $0000		; Set cursor to 0,0
	ld (CURSOR_X), hl

	ld hl, Header
	call DispMenu		; Print the header string which we stored at hl

        ld hl, $0700            ; Set the cursor to 0,7- down one row
	ld (CURSOR_X), hl

	xor a				; xor a does the same thing as ld a, 0 but 1 byte smaller
	ld bc, $FFFF		; Store the highest 16 bit number we can into bc, which is used in cpir or cpdr as the farthest we go back
	ld hl, (CurStAddr)	; Load hl with where we are at right now
	ret

Next:
	call SetGraph		; Go to setgraph

	cpir				; Look forward for the next 0

	ld (CurStAddr), hl	; Save the address we found
	ld a, 255			; 255 is what to check for
	cp (hl)			; See if we landed on a 255, which means we are at the end
	jr z, Prev			; Yes, so go to prev to correct for the error

	call DispMenu		; Otherwise, display the entry

	jr KeyLoop			; Go back

Prev:
	call SetGraph		; Go to setgraph

	dec hl
	dec hl			; Dec the address twice to get before the leading zero
	cpdr				; Look backward for the next 0
	inc hl
	inc hl			; Inc the address twice to get to the start of the entry

	ld (CurStAddr), hl	; Save the address we found
	ld a, 255			; 255 is what to check for
	cp (hl)			; See if we landed on a 255, which means we are at the beginning
	jr z, Next			; Yes, so go to next to correct for the error

	call DispMenu		; Otherwise, display the entry

	jr KeyLoop			; Go back

DispMenu:				; If you do a ROM_CALL a lot, it is smaller to do it this way. 3 times is the break even, I think.
	ROM_CALL(D_ZM_STR)	; Display a string
	ret				; End function

Header:
        .db "Name  Sym  Mass  Num",0

        .db 255,0
Data:
        .db     "Hydrogen H 1.00794  1",0
        .db     "Lithium Li 6.941  3",0
        .db     "Sodium Na 22.98977  11",0
        .db     "Potassium K 39.0983  19",0
        .db     "Rubidium Rb 85.4678  37",0
        .db     "Cesium Cs 132.9054  55",0
        .db     "Francium Fr 223  87",0
        .db     "Beryllium Be 9.01218  4",0
        .db     "Magnesium Mg 24.305  12",0
        .db     "Calcium Ca 40.078  20",0
        .db     "Strontium Sr 87.62  38",0
        .db     "Barium Ba 137.33  56",0
        .db     "Radium Ra 226.0254  88",0
        .db     "Lanthanum La 138.9055  57",0
        .db     "Actinium Ac 227.0278  89",0
        .db     "Cerium Ce 140.12  58",0
        .db     "Thorium Th 232.0381  90",0
        .db     "Praseodymium Pr 140.9077  59",0
        .db     "Protactinium Pa 231.0359  91",0
        .db     "Neodymium Nd 144.24  60",0
        .db     "Uranium U 238.0289  92",0
        .db     "Promethium Pm 145  61",0
        .db     "Neptunium Np 237.048  93",0
        .db     "Samarium Sm 150.36  62",0
        .db     "Plutonium Pu 244  94",0
        .db     "Europium Eu 151.96  63",0
        .db     "Americium Am 243  95",0
        .db     "Gadolinium Gd 157.25  64",0
        .db     "Curium Cm 247  96",0
        .db     "Terbium Tb 158.9254  65",0
        .db     "Berkelium Bk 247  97",0
        .db     "Dysprosium Dy 162.50  66",0
        .db     "Californium Cf 251  98",0
        .db     "Holmium Ho 164.9304  67",0
        .db     "Einsteinium Es 252  99",0
        .db     "Erbium Er 167.26  68",0
        .db     "Fermium Fm 257  100",0
        .db     "Thulium Tm 168.9342  69",0
        .db     "Mendelevium Md 258  101",0
        .db     "Ytterbium Yb 173.04  70",0
        .db     "Nobelium No 259  102",0
        .db     "Scandium Sc 44.9559  21",0
        .db     "Yttrium Y 88.9059  39",0
        .db     "Lutetium Lu 174.967  71",0
        .db     "Lawrencium Lr 260  103",0
        .db     "Titanium Ti 47.88  22",0
        .db     "Zirconium Zr 91.224  40",0
        .db     "Hafnium Hf 178.49  72",0
        .db     "Unnilquadium Unq 261  104",0
        .db     "Vanadium V 50.9415  23",0
        .db     "Niobium Nb 92.9064  41",0
        .db     "Tantalum Ta 180.9479  73",0
        .db     "Unnilpentium Unp 262  105",0
        .db     "Chromium Cr 51.996  24",0
        .db     "Molybdenum Mo 95.94  42",0
        .db     "Tungsten W 183.85  74",0
        .db     "Seaborgium Sg 263  106",0
        .db     "Manganese Mn 54.9380  25",0
        .db     "Technetium Tc 98  43",0
        .db     "Rhenium Re 186.207  75",0
        .db     "Unnilseptium Uns 262  107",0
        .db     "Iron Fe 55.847  26",0
        .db     "Ruthenium Ru 101.07  44",0
        .db     "Osmium Os 190.2  76",0
        .db     "Unniloctium Uno 265  108",0
        .db     "Cobalt Co 58.9332  27",0
        .db     "Rhodium Rh 102.9055  45",0
        .db     "Iridium Ir 192.22  77",0
        .db     "Unnilennium Une 266  109",0
        .db     "Nickel Ni 58.69  28",0   
        .db     "Palladium Pd 106.42  46",0
        .db     "Platinum Pt 195.08  78",0
        .db     "Copper Cu 63.546  29",0
        .db     "Silver Ag 107.8682  47",0
        .db     "Gold Au 196.9665  79",0
        .db     "Zinc Zn 65.39  30",0
        .db     "Cadmium Cd 112.41  48",0
        .db     "Mercury Hg 200.59  80",0
        .db     "Boron B 10.81  5",0
        .db     "Aluminum Al 26.98154  13",0
        .db     "Gallium Ga 69.72  31",0
        .db     "Indium In 114.82  49",0
        .db     "Thallium Tl 204.38  81",0
        .db     "Carbon C 12.011  6",0
        .db     "Silicon Si 28.0855  14",0
        .db     "Germanium Ge 72.61  32",0
        .db     "Tin Sn 118.710  50",0
        .db     "Lead Pb 207.2  82",0
        .db     "Nitrogen N 14.0067  7",0
        .db     "Phosphorous P 30.97376  15",0
        .db     "Arsenic As 74.9216  33",0
        .db     "Antimony Sb 121.757  51",0
        .db     "Bismuth Bi 208.9804  83",0
        .db     "Oxygen O 15.9994  8",0
        .db     "Sulfur S 32.066  16",0
        .db     "Selenium Se 78.96  34",0
        .db     "Tellurium Te 127.60  52",0
        .db     "Polonium Po 209  84",0
        .db     "Fluorine F 18.998403  9",0
        .db     "Chlorine Cl 35.453  17",0
        .db     "Bromine Br 79.904  35",0
        .db     "Iodine I 126.9045  53",0
        .db     "Astatine At 210  85",0
        .db     "Helium He 4.00260  2",0
        .db     "Neon Ne 20.1797  10",0
        .db     "Argon Ar 39.948  18",0
        .db     "Krypton Kr 83.80  36",0
        .db     "Xenon Xe 131.29  54",0
        .db     "Radon Rn 222  86",0
        .db 255,0
.end
